/*****************************************************************************
**  This is part of the CTSim program
**  Copyright (C) 1983-2000 Kevin Rosenberg
**
**  $Id: syserror.cpp,v 1.3 2000/06/19 19:04:05 kevin Exp $
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License (version 2) as
**  published by the Free Software Foundation.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

#include <iostream>
#include <exception>
#include <stdexcept>
#include <stdarg.h>
#include <ctype.h>
#include "ctsupport.h"


/* NAME
 *   sys_error			System error handler
 *
 * SYNOPSIS
 *   sys_error (severity, msg, args . . .)
 *   int severity		Severity of error
 *   char *msg			Error message
 *   args			Argument list, direct transfer to printf stack
 *				Can take 24 byte transfer
 */

static int errorlevel = ERR_WARNING;	/* Set error reporting level */


void sys_error (int severity, const char *msg, ...)
{
  va_list arg;

  va_start(arg, msg);

  sys_verror (severity, msg, arg);

  va_end(arg);
}

static int nErrorCount = 0;
const static int MAX_ERROR_COUNT = 20;


void sys_verror (int severity, const char *msg, va_list arg)
{
  if (severity < errorlevel)
    return;			/* ignore error if less than max level */

  nErrorCount++;
  if (severity != ERR_FATAL) {
    if (nErrorCount > MAX_ERROR_COUNT)
      return;
    else if (nErrorCount == MAX_ERROR_COUNT) {
      cout << "*****************************************************************" << endl;
      cout << "***     M A X I M U M  E R R O R  C O U N T  R E A C H E D    ***" << endl;
      cout << "***                                                           ***" << endl;
      cout << "***           No further errors will be reported              ***" << endl;
      cout << "*****************************************************************" << endl;
      return;
    }
  }
    
  switch (severity) {
  case ERR_FATAL:
    cout << "FATAL ERROR: ";
    break;
  case ERR_SEVERE:
    cout << "SEVERE ERROR: ";
    break;
  case ERR_WARNING:
    cout << "WARNING ERROR: ";
    break;
  default:
    sys_error (ERR_FATAL, "illegal error code #%d  [sys_error]", severity);
  }
  
  char errStr[512];
  vsnprintf (errStr, sizeof(errStr), msg, arg);
  cout << errStr << endl;
  
  if (severity == ERR_FATAL)
    throw runtime_error (errStr);
  
#if INTERACTIVE_ERROR_DISPLAY
  cout << "A - Abort  C - Continue  W - Turn off warnings? ";
  //  fflush(stderr);
  do 
   {
      int c = cio_kb_waitc("AaBbCcWw", TRUE);	  /* get code from keyboard */
      c = tolower (c);
      fputc (c, stderr);
      fputc (NEWLINE, stderr);
      
      if (c == 'a')
	exit (1);
      else if (c == 'c')
	return;
      else if (c == 'w') 
	{
	  sys_error_level (ERR_SEVERE);	/* report severe & fatal errors */
	  break;
	}
    } while (TRUE);
#endif
}


/* NAME
 *   sys_error_level			Set error reporting level
 *
 * SYNOPSIS
 *   sys_error_level (severity)
 *   int severity		Report all error as serious as severity and beyond
 *
 * DESCRIPTION
 *   Causes the system to ignore all error below the level of severity
 *   For example, if severity == ERR_SEVERE, then report severe & fatal
 *   error and ignore warnings
 */

void 
sys_error_level (int severity)
{
  if (severity == ERR_FATAL ||
      severity == ERR_SEVERE ||
      severity == ERR_WARNING)
    errorlevel = severity;
  else
    errorlevel = ERR_WARNING;
}

