/*****************************************************************************
** FILE IDENTIFICATION
**
**   Name:          if-1.cpp
**   Purpose:       Manipulate a single image file
**   Programmer:    Kevin Rosenberg
**   Date Started:  Aug 1984
**
**  This is part of the CTSim program
**  Copyright (C) 1983-2000 Kevin Rosenberg
**
**  $Id: if-1.cpp,v 1.8 2000/06/19 17:58:13 kevin Exp $
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License (version 2) as
**  published by the Free Software Foundation.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

/* FILE
 *   if-1.c             Filter a single IF file
 */

#include "ct.h"

enum {O_LOG, O_EXP, O_SQRT, O_SQR, O_INVERT, O_VERBOSE, O_HELP, O_VERSION};

static struct option my_options[] =
{
  {"invert", 0, 0, O_INVERT},
  {"verbose", 0, 0, O_VERBOSE},
  {"log", 0, 0, O_LOG},
  {"exp", 0, 0, O_EXP},
  {"sqr", 0, 0, O_SQR},
  {"sqrt", 0, 0, O_SQRT},
  {"help", 0, 0, O_HELP},
  {"version", 0, 0, O_VERSION},
  {0, 0, 0, 0}
};

void 
if1_usage (const char *program)
{
  cout << "usage: " << fileBasename(program) << " infile outfile [OPTIONS]" << endl;
  cout << "Generate a IF file from a IF file" << endl;
  cout << endl;
  cout << "     --invert   Invert image" << endl;
  cout << "     --log      Natural logrithm of image" << endl;
  cout << "     --exp      Natural exponential of image" << endl;
  cout << "     --sqr      Square of image" << endl;
  cout << "     --sqrt     Square root of image" << endl;
  cout << "     --verbose  Verbose modem" << endl;
  cout << "     --version  Print version" << endl;
  cout << "     --help     Print this help message" << endl;
}

int 
if1_main (int argc, char *const argv[])
{
  ImageFile *im_in;
  ImageFile *im_out;
  char *in_file;
  char *out_file;
  int opt_verbose = 0;
  int opt_invert = 0;
  int opt_log = 0;
  int opt_exp = 0;
  int opt_sqr = 0;
  int opt_sqrt = 0;

  while (1)
    {
      int c = getopt_long (argc, argv, "", my_options, NULL);
      
      if (c == -1)
	break;
      
      switch (c)
	{
	case O_INVERT:
	  opt_invert = 1;
	  break;
	case O_LOG:
	  opt_log = 1;
	  break;
	case O_SQR:
	  opt_sqr = 1;
	  break;
	case O_SQRT:
	  opt_sqrt = 1;
	  break;
	case O_EXP:
	  opt_exp = 1;
	  break;
	case O_VERBOSE:
	  opt_verbose = 1;
	  break;
        case O_VERSION:
#ifdef VERSION
	  cout << "Version " <<  VERSION << endl;
#else
          cout << "Unknown version number" << endl;
#endif
	  return (0);
	case O_HELP:
	case '?':
	  if1_usage(argv[0]);
	  return (0);
	default:
	  if1_usage(argv[0]);
	  return (1);
	}
    }

  if (optind + 2 != argc)
    {
      if1_usage(argv[0]);
      return (1);
    }
  
  in_file = argv[optind];
  out_file = argv[optind + 1];


  string histString;

  if (opt_invert || opt_log || opt_exp || opt_sqr || opt_sqrt) {
    int ix, iy;

    im_in = new ImageFile (in_file);
    im_in->fileRead ();
    int nx = im_in->nx();
    int ny = im_in->ny();
    im_out = new ImageFile (out_file, nx, ny);
    im_out->fileCreate ();

    ImageFileArray vIn = im_in->getArray();
    ImageFileArray vOut = im_out->getArray();

    if (opt_invert) {
      for (ix = 0; ix < nx; ix++)
        for (iy = 0; iy < ny; iy++)
          vOut[ix][iy] = - vIn[ix][iy];
      histString = "Invert transformation";
    }
    if (opt_log) {
      for (ix = 0; ix < nx; ix++)
        for (iy = 0; iy < ny; iy++)
          vOut[ix][iy] = log (vIn[ix][iy]);
      histString = "Log transformation";
    }
    if (opt_exp) {
      for (ix = 0; ix < nx; ix++)
        for (iy = 0; iy < ny; iy++)
          vOut[ix][iy] = exp (vIn[ix][iy]);
      histString = "Exp transformation";
    }
    if (opt_sqr) {
      for (ix = 0; ix < nx; ix++)
        for (iy = 0; iy < ny; iy++)
          vOut[ix][iy] = vIn[ix][iy] * vIn[ix][iy];
      histString = "Sqr transformation";
    }
    if (opt_sqrt) {
      for (ix = 0; ix < nx; ix++)
        for (iy = 0; iy < ny; iy++)
          vOut[ix][iy] = sqrt (vIn[ix][iy]);
      histString = "Sqrt transformation";
    }

    im_out->arrayDataWrite ();
    im_out->labelsCopy (*im_in);
    im_out->labelAdd (Array2dFileLabel::L_HISTORY, histString.c_str());
    im_out->fileClose ();
  }

  return (0);
}

#ifndef NO_MAIN
int 
main (int argc, char *const argv[])
{
  int retval = 1;

  try {
    retval = if1_main(argc, argv);
  } catch (exception e) {
    cerr << "Exception: " << e.what() << endl;
  } catch (...) {
    cerr << "Unknown exception" << endl;
  }

  return (retval);
}
#endif

